@@ -2,7 +2,7 @@
  * greedy.cpp
  *
  * @author Alain Hauser
- * $Id: greedy.cpp 248 2014-03-03 11:27:22Z alhauser $
+ * $Id: greedy.cpp 256 2014-04-09 11:54:39Z alhauser $
  */
 
 #include "pcalg/greedy.hpp"
@@ -411,41 +411,6 @@ EssentialGraph EssentialGraph::getRepresentative() const
 	return representative;
 }
 
-std::vector<boost::dynamic_bitset<> > EssentialGraph::getAllRepresentatives() const
-{
-	// TODO: implement! Idea:
-	// * get all chain components
-	// * for each chain component, use "one source lemma": each vertex of a chain
-	// compoment can be the unique source; orient edges away from that source, consider
-	// it as "intervention target". Go on iteratively.
-
-
-	// Get all chain components
-	boost::dynamic_bitset<> notVisited(getVertexCount());
-	notVisited.set();
-	std::vector<std::vector<uint> > chainComponents;
-	std::set<uint> chainComp;
-	std::set<uint>::iterator vi;
-	uint v;
-	while ((v = notVisited.find_first()) < getVertexCount()) {
-		chainComp = getChainComponent(v);
-		chainComponents.push_back(std::vector<uint>());
-		chainComponents.back().reserve(chainComp.size());
-		for (vi = chainComp.begin(); vi != chainComp.end(); ++vi) {
-			chainComponents.back().push_back(*vi);
-			notVisited.reset(*vi);
-		}
-	}
-
-	// Find all "representatives" on the chain components
-	EssentialGraph subgraph;
-	std::vector<std::vector<boost::dynamic_bitset<> > > orientations(chainComponents.size());
-	for (std::size_t i = 0; i < chainComponents.size(); ++i) {
-		subgraph = inducedSubgraph(chainComponents[i].begin(), chainComponents[i].end());
-		// TODO go on!!!
-	}
-}
-
 void EssentialGraph::enableCaching()
 {
 	if (!_doCaching) {
